﻿/*
 * Copyright (c) 2008-2013, RF-Embedded GmbH
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using CSrfeReaderInterface;
using CSrfeReaderInterface.protocol;
using System.Net;
using System.Threading;
using CSrfeReaderInterface.rfe.protocol;

namespace Test
{
    class Program
    {
        static CSrfeProtocolHandler _ph = null;

        static void Main(string[] args)
        {
            Console.WriteLine("");
            Console.WriteLine("=============================================================");
            Console.WriteLine("|| Welcome to the RF-Embedded Open Source Protocol-Handler ||");
            Console.WriteLine("=============================================================");
            
            CSrfeReaderInterface.device.IProtocolDeviceInterface dev = null;
            

            // Get the serial device where the reader is connected to
            while (true)
            {

                Console.WriteLine("\n\n"); 
                Console.WriteLine("Select connection type: 's' for serial 't' for tcp");

                string connectionType = Console.ReadLine();

                if (connectionType == "s")
                {
                    Console.WriteLine("Available COM Ports: ");
                    // get all available port names
                    string[] theSerialPortNames = System.IO.Ports.SerialPort.GetPortNames();

                    // print them out with index to select
                    for (int i = 0; i < theSerialPortNames.Length; i++)
                    {
                        Console.WriteLine("\t [ {0:d} ]: {1}", i, theSerialPortNames[i]);
                    }
                    Console.Write("Please select the index of the COM Port of the reader (Enter 'q' to Quit): > ");
                    string port = Console.ReadLine();
                    Console.WriteLine("");

                    if (port == "q")
                        return;

                    // try to open the serial port
                    dev = new Test.SerialDevice(theSerialPortNames[Convert.ToInt32(port)]);
                    if (!dev.Open())
                    {
                        Console.WriteLine("Could not open the COM-Port.");
                    }
                    else
                    {
                        Console.WriteLine("The COM-Port was open successfully");
                        break;
                    }
                }
                //in case of tcp:
                else if (connectionType == "t")
                {
                    Console.WriteLine("Please type ip address");
                    string ipAddressString = Console.ReadLine();

                    Console.WriteLine("Please type port");
                    string portString = Console.ReadLine();

                    try
                    {
                        int port = int.Parse(portString);

                        IPAddress ip = IPAddress.Parse(ipAddressString);
                        dev = new TCPDevice(ip, port);
                        //try to connect over tcp
                        if (!dev.Open())
                        {
                            Console.WriteLine("Could not open the TCP-Socket.");
                        }
                        else
                        {
                            Console.WriteLine("The TCP-Socket was open successfully");
                            //Continue by breaking while(true)
                            break;
                        }
                    }
                    catch (FormatException /*formatException*/)
                    {
                        Console.WriteLine("Invalid address or port input");
                    }              

                }
            }


            // create trace and turn off 
            Global.m_tracer = new Test.ConsoleTrace();
            Global.m_tracer.TraceLevel = 0;

            // create new instance of the protocol handler with the selected serial device
            CSrfeProtocolHandler ph = null;
            while (true)
            {
                Console.WriteLine("\n\n");
                Console.WriteLine("Select protocol implementation: 's' for standard, 'p' for PUR, 'a' for AUR");
                string readerType = Console.ReadLine();

                if (readerType == "s")
                {
                    ph = new CSrfeProtocolHandler(dev);
                    break;
                }
                else if(readerType == "p")
                {
                    ph = new CSrfePURprotocolHandler(dev);
                    break;
                }
                else if (readerType == "a")
                {
                    ph = new CSrfeAURprotocolHandler(dev);
                    break;
                }
            }
            
            _ph = ph;

            // connect events to the local event handler
            ph.HeartBeat += new CSrfeProtocolHandler.HeartBeatHandler(onHeratBeat);
            ph.StateChanged += new CSrfeProtocolHandler.StateChangedHandler(onStateChanged);
            ph.GpioValuesChanged += new CSrfeProtocolHandler.GpioValuesChangedHandler(onGpioValuesChanged);
            ph.CyclicInventory += new CSrfeProtocolHandler.CyclicInventoryHandler(onCyclicInventory);


            // print out menu and process what is selected
            while (true)
            {
                Console.Clear();
                Console.WriteLine("");
                Console.WriteLine("+------------------------ MENU ----------------------------+");
                Console.WriteLine("|    q = Quit                                              |");
                Console.WriteLine("|    1 = Query Reader Information                          |");
                Console.WriteLine("|    2 = Test Attenuation Settings                         |");
                Console.WriteLine("|    3 = Test Frequency Settings                           |");
                Console.WriteLine("|    4 = Test Sensitivity Settings                         |");
                Console.WriteLine("|    5 = Test Heartbeat                                    |");
                Console.WriteLine("|    6 = Test GPIO - Output                                |");
                Console.WriteLine("|    7 = Test GPIO - Input                                 |");
                Console.WriteLine("|    8 = Start Inventory                                   |");
                Console.WriteLine("|    9 = Do SinglInventory                                 |");
                Console.WriteLine("+-PUR-----------------------------------------------------+");
                Console.WriteLine("|   21 = Start RSSI Inventory                              |");
                Console.WriteLine("|   22 = Try to Read TID                                   |");
                Console.WriteLine("|   23 = Try to Read/Write User Mem                        |");
                Console.WriteLine("+-AN-PUR---------------------------------------------------+");
                Console.WriteLine("|   31 = AN001 - PUR - Test Read TID of First Tag - Slow   |");
                Console.WriteLine("|   32 = AN001 - PUR - Test Read TID of First Tag - Fast   |");
                Console.WriteLine("|   33 = AN001 - PUR - Test Read TID of First Tag - Handle |");
                Console.WriteLine("+----------------------------------------------------------+\n");

                Console.Write("Please choose: > ");
                string id = Console.ReadLine();

                Console.WriteLine("");

                if (id == "q")
                {
                    Console.WriteLine("Exit");
                    break;
                }
                else if (id == "1")
                    test_ReaderInfo(ph);
                else if (id == "2")
                    test_Attenuation(ph);
                else if (id == "3")
                    test_Frequency(ph);
                else if (id == "4")
                    test_Sensitivity(ph);
                else if (id == "5")
                    test_Heartbeat(ph);
                else if (id == "6")
                    test_GPIO_Output(ph);
                else if (id == "7")
                    test_GPIO_Input(ph);
                else if (id == "8")
                    test_Inventory(ph);
                else if (id == "9")
                    test_SingleInventory(ph);

                else if (id == "21")
                    test_InventoryRSSI(ph);
                else if (id == "22")
                    test_ReadTID(ph);
                else if (id == "23")
                    test_ReadWriteUser(ph);

                else if (id == "31")
                    test_AN001_ReadTIDFirstTag_Slow(ph);
                else if (id == "32")
                    test_AN001_ReadTIDFirstTag_Fast(ph);
                else if (id == "33")
                    test_AN001_ReadTIDFirstTag_Handle(ph);
                else
                    Console.WriteLine("Not yet implemented");

                Console.WriteLine("\n\n\n");
                Console.WriteLine("Press enter to proceed...");
                Console.ReadLine();
            }

            dev.Close();
        }

        #region Test Methods

        #region Common

        public static void test_ReaderInfo(CSrfeProtocolHandler ph)
        {
            bool ok = false;

            // get reader id
            uint readerId = 0;
            ok = ph.getReaderID(out readerId);
            if (!ok)
                Console.WriteLine("ERROR: Could not get ReaderID");

            // get reader type
            uint readerType = 0;
            ok = ph.getReaderType(out readerType);
            if (!ok)
                Console.WriteLine("ERROR: Could not get ReaderType");

            // get hardware revision
            uint hwRev = 0;
            ok = ph.getHardwareRevision(out hwRev);
            if (!ok)
                Console.WriteLine("ERROR: Could not get HardwareRevision");

            // get software revision
            uint swRev = 0;
            ok = ph.getSoftwareRevision(out swRev);
            if (!ok)
                Console.WriteLine("ERROR: Could not get SoftwareRevision");

            // get bootloader revision
            uint blRev = 0;
            ok = ph.getBootloaderRevision(out blRev);
            if (!ok)
                Console.WriteLine("ERROR: Could not get BootloaderRevision");

            // get current system
            Constants.eRFE_CURRENT_SYSTEM system = 0;
            ok = ph.getCurrentSystem(out system);
            if (!ok)
                Console.WriteLine("ERROR: Could not get CurrentSystem");

            // get current state
            Constants.eRFE_CURRENT_READER_STATE state = 0;
            ok = ph.getCurrentState(out state);
            if (!ok)
                Console.WriteLine("ERROR: Could not get CurrentState");

            // get status register
            ulong statusReg = 0;
            ok = ph.getStatusRegister(out statusReg);
            if (!ok)
                Console.WriteLine("ERROR: Could not get StatusRegister");

            // print out results
            Console.WriteLine("Reader Information:");
            Console.WriteLine("\t -> ReaderID       = 0x{0:X08}", readerId);
            Console.WriteLine("\t -> ReaderType     = 0x{0:X08}", readerType);
            Console.WriteLine("\t -> HardwareRev    = 0x{0:X08}", hwRev);
            Console.WriteLine("\t -> SoftwareRev    = 0x{0:X08}", swRev);
            Console.WriteLine("\t -> BootloaderRev  = 0x{0:X08}", blRev);
            Console.WriteLine("\t -> Current System = {0}", system.ToString());
            Console.WriteLine("\t -> Current State  = {0}", state.ToString());
            Console.WriteLine("\t -> StatusRegister = 0x{0:X016}", statusReg);
        }

        public static void test_Attenuation(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Testing attenuation settings:");

            ushort attMax, attCur, attTemp;

            // get attenuation values
            Console.WriteLine("\t -> 1, Reading attenuation settings:");
            ph.getAttenuation(out attMax, out attCur);
            Console.WriteLine("\t\t Attenuation Max={0} Current={1}", attMax, attCur);
            Console.WriteLine("");

            // set attenuation to fix value 10
            Console.WriteLine("\t -> 2, Setting attenuation settings:");
            ph.setAttenuation(10);
            Console.WriteLine("\t\t Set Attenuation to 10");
            Console.WriteLine("");

            // get attenuation settings and check if the fix value is set
            Console.WriteLine("\t -> 3, Reading attenuation settings:");
            ph.getAttenuation(out attMax, out attTemp);
            Console.WriteLine("\t\t Attenuation Max={0} Current={1}", attMax, attTemp);
            Console.WriteLine("\t\t Current attenuation {0} == 10?", attTemp);
            if (attTemp != 10)
            {
                Console.WriteLine("ERROR------------------ Set Attenuation is not the Current Attenuation");
                return;
            }
            Console.WriteLine("\t\t OK\n");

            // retore attenuation to the previous value
            Console.WriteLine("\t -> 4, Restore attenuation settings:");
            ph.setAttenuation(attCur);
            Console.WriteLine("\t\t Set Attenuation to 0");
            Console.WriteLine("");

            // check the set values again
            Console.WriteLine("\t -> 5, Reading attenuation settings:");
            ph.getAttenuation(out attMax, out attCur);
            Console.WriteLine("\t\t Attenuation Max={0} Current={1}", attMax, attCur);
            Console.WriteLine("");
        }

        public static void test_Frequency(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Testing frequency settings:");

            byte mode, maxCount;
            List<uint> frequ;
            List<uint> tempFrequ;

            // get current frequency table
            Console.WriteLine("\t -> 1, Reading frequency settings:");
            ph.getFrequency(out mode, out maxCount, out frequ);
            Console.WriteLine("\t\t FrequencyTable Mode={0} Max={1} Current={2}", mode, maxCount, frequ.Count());
            for (int i = 0; i < frequ.Count(); i++)
            {
                Console.WriteLine("\t\t\t {0} = {1}", i, frequ[i]);
            }
            Console.WriteLine("");

            // create new frequency table and set this
            Console.WriteLine("\t -> 2, Setting frequency settings:");
            List<uint> newFreq = new List<uint>( new uint[] { 855000, 856000, 857000 });
            ph.setFrequency(mode, newFreq);
            Console.WriteLine("\t\t Set FrequencyTable to Mode={0}", mode);
            for (int i = 0; i < newFreq.Count(); i++)
            {
                Console.WriteLine("\t\t\t {0} = {1}", i, newFreq[i]);
            }
            Console.WriteLine("");

            // retrieve the table again and check if the retireved table is the same like the set
            Console.WriteLine("\t -> 3, Reading frequency settings:");
            ph.getFrequency(out mode, out maxCount, out tempFrequ);
            Console.WriteLine("\t\t FrequencyTable Mode={0} Max={1} Current={2}", mode, maxCount, tempFrequ.Count());
            for (int i = 0; i < tempFrequ.Count(); i++)
            {
                Console.WriteLine("\t\t\t {0} = {1}", i, tempFrequ[i]);
            }
            Console.WriteLine("\t\t Last set Frequency Table == Read Frequency Table?");
            if (!newFreq.TrueForAll(tempFrequ.Contains) || !tempFrequ.TrueForAll(newFreq.Contains))
            {
                Console.WriteLine("ERROR------------------ Previous Set Frequency Table is not the Current Frequency Table!!");
                return;
            }
            Console.WriteLine("\t\t OK\n");

            // restore the previous frequency table
            Console.WriteLine("\t -> 4, Restoring frequency settings:");
            ph.setFrequency(mode, frequ);
            Console.WriteLine("\t\t Set FrequencyTable to Mode={0}", mode);
            for (int i = 0; i < frequ.Count(); i++)
            {
                Console.WriteLine("\t\t\t {0} = {1}", i, frequ[i]);
            }
            Console.WriteLine("");

            // recheck the frequency table
            Console.WriteLine("\t 5, Reading frequency settings:");
            ph.getFrequency(out mode, out maxCount, out tempFrequ);
            Console.WriteLine("\t\t FrequencyTable Mode={0} Max={1} Current={2}", mode, maxCount, tempFrequ.Count());
            for (int i = 0; i < tempFrequ.Count(); i++)
            {
                Console.WriteLine("\t\t\t {0} = {1}", i, tempFrequ[i]);
            }
            Console.WriteLine("");
        }

        public static void test_Sensitivity(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Testing sensitivity settings:");

            short maxSens, minSens, curSens, temSens, actSens;
 
            // get current sensitivity
            Console.WriteLine("\t -> 1, Reading sensitivity settings:");
            ph.getSensitivity(out maxSens, out minSens, out curSens);
            Console.WriteLine("\t\t Sensitivity Max={0} Min={1} Current={2}", maxSens, minSens, curSens);
            Console.WriteLine("");

            // set minimum sensitivity and read out the actual set value
            Console.WriteLine("\t -> 2, Setting sensitivity settings to MIN:");
            ph.setSensitivity(minSens, out actSens);
            Console.WriteLine("\t\t Sensitivity Min={0} Actual={1}", minSens, actSens);
            Console.WriteLine("");

            // retrieve the current sensitivity and check if the current value is the same like the actual set one 
            Console.WriteLine("\t -> 3, Reading sensitivity settings:");
            ph.getSensitivity(out maxSens, out minSens, out temSens);
            Console.WriteLine("\t\t Sensitivity Max={0} Min={1} Current={2}", maxSens, minSens, temSens);
            Console.WriteLine("\t\t Current Sensitivity {0} == {1} Actual Set Sensitivirty?", temSens, actSens);
            if (actSens != temSens)
            {
                Console.WriteLine("ERROR------------------ Actual Set Sensitivity is not the Current Sensitivity");
                return;
            }
            Console.WriteLine("\t\t OK\n");

            // set maximum sensitivity and read out the actual set value
            Console.WriteLine("\t -> 4, Setting sensitivity settings to MAX:");
            ph.setSensitivity(maxSens, out actSens);
            Console.WriteLine("\t\t Sensitivity Max={0} Actual={1}", maxSens, actSens);
            Console.WriteLine("");


            // retrieve the current sensitivity and check if the current value is the same like the actual set one 
            Console.WriteLine("\t -> 5, Reading sensitivity settings:");
            ph.getSensitivity(out maxSens, out minSens, out temSens);
            Console.WriteLine("\t\t Sensitivity Max={0} Min={1} Current={2}", maxSens, minSens, temSens);
            Console.WriteLine("\t\t Current Sensitivity {0} == {1} Actual Set Sensitivirty?", temSens, actSens);
            if (actSens != temSens)
            {
                Console.WriteLine("ERROR------------------ Actual Set Sensitivity is not the Current Sensitivity");
                return;
            }
            Console.WriteLine("\t\t OK\n");

            // restore the previous set value
            Console.WriteLine("\t -> 6, Restore sensitivity settings:");
            ph.setSensitivity(curSens, out actSens);
            Console.WriteLine("\t\t Sensitivity Cur={0} Actual={1}", curSens, actSens);
            Console.WriteLine("\t\t Current Sensitivity {0} == {1} Actual Set Sensitivirty?", curSens, actSens);
            if (actSens != curSens)
            {
                Console.WriteLine("ERROR------------------ Actual Set Sensitivity is not the Current Sensitivity");
                return;
            }
            Console.WriteLine("\t\t OK\n");
        }

        public static void test_Heartbeat(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Testing Heartbeat:");

            // turn on heartbeat with an interval of 250ms
            Console.WriteLine("\t -> 1, Setting Heartbeat ON with interval 250ms (Press Enter to Stop)");
            lastHeartbeat = DateTime.Now;
            ph.setHeartBeat(true, 250);

            // wait for <enter>
            Console.ReadLine();

            // turn off heartbeat
            Console.WriteLine("\t -> 2, Setting Heartbeat OFF");
            ph.setHeartBeat(false, 250);
        }

        public static void test_GPIO_Output(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Test GPIO - Output");

            ulong mask, output, input;
            if (!ph.getGPIOCaps(out mask, out output, out input))
            {
                Console.WriteLine("ERROR: Could not get GPIO Caps: " + ph.LastReturnCode.ToString());
                return;
            }

            Console.WriteLine("Setting each pin output.");
            if (!ph.setGPIODirection(output))
            {
                Console.WriteLine("ERROR: Could not set GPIO direction: " + ph.LastReturnCode.ToString());
                return;
            }

            Console.WriteLine();
            ulong test_mask = 0;
            for (int i = 0; i < 32; i++)
            {
                test_mask = (ulong)(((int)1) << i);
                if ((test_mask & output) == 0)
                    continue;

                Console.WriteLine("Setting pin: 0x{0:X08}", test_mask);
                if (!ph.setGPIO(test_mask))
                {
                    Console.WriteLine("ERROR: Could not set GPIO: " + ph.LastReturnCode.ToString());
                    break;
                }

                Console.WriteLine("Press enter to clear the Pin and to proceed");
                Console.ReadLine();

                if (!ph.clearGPIO(test_mask))
                {
                    Console.WriteLine("ERROR: Could not clear GPIO: " + ph.LastReturnCode.ToString());
                    break;
                }
            }

            Console.WriteLine("Setting all Pins low.");
            if (!ph.clearGPIO(mask))
            {
                Console.WriteLine("ERROR: Could not clear GPIO: " + ph.LastReturnCode.ToString());
                return;
            }
        }

        public static void test_GPIO_Input(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Test GPIO - Input");

            ulong mask, output, input;
            if (!ph.getGPIOCaps(out mask, out output, out input))
            {
                Console.WriteLine("ERROR: Could not get GPIO Caps: " + ph.LastReturnCode.ToString());
                return;
            }

            Console.WriteLine("Setting each input pin as input: 0x{0:X08}", input);
            if (!ph.setGPIODirection(~input))
            {
                Console.WriteLine("ERROR: Could not set GPIO direction: " + ph.LastReturnCode.ToString());
                return;
            }

            Console.WriteLine();
            Console.WriteLine("Press enter to clear the Pin and to proceed");
            Console.ReadLine();


            Console.WriteLine("Setting all pins output.");
            if (!ph.clearGPIO(output))
            {
                Console.WriteLine("ERROR: Could not clear GPIO: " + ph.LastReturnCode.ToString());
                return;
            }
        }

        public static void test_Inventory(CSrfeProtocolHandler ph)
        {
            Console.WriteLine("Testing Cyclic Inventory:");

            // turn on cyclic inventory
            Console.WriteLine("\t -> 1, Starting Cyclic Inventory (Press Enter to Stop)");
            tagReadCount = 0;
            DateTime startTime = DateTime.Now;
            ph.setCyclicInventory(true);
            
            // wait for <enter>
            Console.ReadLine();
            
            // turn off cyclic inventory and calculate read rate
            ph.setCyclicInventory(false);
            double secs = DateTime.Now.Subtract(startTime).TotalSeconds;
            Console.WriteLine("\t -> 2, Stopped Cyclic Inventry with a ReadRate of {0} reads/sec", tagReadCount/secs);
        }
        
        public static void test_SingleInventory(CSrfeProtocolHandler ph)
        {
            // do a single inventory and print out the list
            Console.WriteLine("Do SingleInventory");
            List<byte[]> epcList;
            ph.doSingleInventory(out epcList);
            Console.WriteLine("\t\t -> Found {0} Tags", epcList.Count);
            foreach (byte[] epc in epcList)
                Console.WriteLine("\t\t TagId: {0}", BitConverter.ToString(epc));
        }

        #endregion

        #region PUR

        public static void test_InventoryRSSI(CSrfeProtocolHandler ph)
        {
            CSrfePURprotocolHandler pur_ph = _ph as CSrfePURprotocolHandler;
            if (pur_ph == null)
            {
                Console.WriteLine("Unable to test. Protocol implementation is not valid. Should be PUR.");
                return;
            } 
            
            Console.WriteLine("Testing Cyclic Inventory with RSSI:");

            // switch sending of rssi on for PUR reader
            Console.WriteLine("\t -> 1, Switching RSSI On");
            ph.setParam(0x02, new byte[] { 0x01 });
            
            // turn on cyclic inventory
            Console.WriteLine("\t -> 2, Starting Cyclic Inventory (Press Enter to Stop)");
            tagReadCount = 0;
            DateTime startTime = DateTime.Now;
            ph.setCyclicInventory(true);

            // wait for <enter>
            Console.ReadLine();

            // turn off cyclic inventory and calculate read rate
            ph.setCyclicInventory(false);
            double secs = DateTime.Now.Subtract(startTime).TotalSeconds;
            Console.WriteLine("\t -> 3, Stopped Cyclic Inventry with a ReadRate of {0} reads/sec", tagReadCount / secs);

            // switch sending of rssi off
            Console.WriteLine("\t -> 4, Switching RSSI On");
            ph.setParam(0x02, new byte[] { 0x00 });
        }
 
        public static void test_ReadTID(CSrfeProtocolHandler ph)
        {
            CSrfePURprotocolHandler pur_ph = _ph as CSrfePURprotocolHandler;
            if (pur_ph == null)
            {
                Console.WriteLine("Unable to test. Protocol implementation is not valid. Should be PUR.");
                return;
            } 
            
            Console.WriteLine("Trying to read tid register:");

            // do a single inventory and print out the tag list with an index to select one tag
            Console.WriteLine("\t -> 1, Searching for tags:");
            List<byte[]> epcList;
            ph.doSingleInventory(out epcList);
            Console.WriteLine("\t\t Found {0} Tags", epcList.Count);
            if (epcList.Count() > 0)
            {
                for (int i = 0; i < epcList.Count(); i++)
                    Console.WriteLine("\t\t\t [{0}]: {1}", i+1, BitConverter.ToString(epcList[i]));
            }
            else
            {
                Console.WriteLine("\t\t Found 0 Tags, stopping the test...");
                return;
            }
            Console.WriteLine("");

            // read the selected index
            Console.Write("\t -> 2, Select a tag by index: > ");
            int k = Console.ReadLine()[0];
            int index = k - '0' - 1;
            if (index < 0 || index >= epcList.Count())
            {
                Console.WriteLine("\t\t Selected unkown tag, stopping the test...");
                return;
            }
            Console.WriteLine("");

            // try to read the first 4 bytes of the TID membank of the selected tag
            Console.WriteLine("\t -> 3, Trying to read first 4 bytes of TID register:");
            byte[] passwd = new byte[4] { 0, 0, 0, 0 };
            byte[] data;
            if (ph.readFromTag(epcList[index], (byte)0x02, 0, passwd, 4, out data))
            {
                Console.WriteLine("\t\t Read succeeded, read data: {0}", BitConverter.ToString(data));
            }
            else
            {
                Console.WriteLine("\t\t Read failed");
            }
            Console.WriteLine("");
        }

        public static void test_ReadWriteUser(CSrfeProtocolHandler ph)
        {
            CSrfePURprotocolHandler pur_ph = _ph as CSrfePURprotocolHandler;
            if (pur_ph == null)
            {
                Console.WriteLine("Unable to test. Protocol implementation is not valid. Should be PUR.");
                return;
            } 
            
            Console.WriteLine("Trying to read and write user register:");

            // do a single inventory and print out the tag list with an index to select one tag
            Console.WriteLine("\t -> 1, Searching for tags:");
            List<byte[]> epcList;
            ph.doSingleInventory(out epcList);
            Console.WriteLine("\t\t Found {0} Tags", epcList.Count());
            if (epcList.Count() > 0)
            {
                for (int i = 0; i < epcList.Count(); i++)
                    Console.WriteLine("\t\t\t [{0}]: {1}", i+1, BitConverter.ToString(epcList[i]));
            }
            else
            {
                Console.WriteLine("\t\t Found 0 Tags, stopping the test...");
                return;
            }
            Console.WriteLine("");

            // read the selected index
            Console.Write("\t -> 2, Select a tag by index: > ");
            int k = Console.ReadLine()[0];
            int index = k - '0' - 1;
            if (index < 0 || index >= epcList.Count())
            {
                Console.WriteLine("\t\t Selected unkown tag, stopping the test...");
                return;
            }
            Console.WriteLine("");

            byte memBank = 0x03;
            byte[] passwd = new byte[4] { 0, 0, 0, 0 };
            byte[] userMemBefore;
            byte[] userMemAfter;
            byte[] old4Bytes = new byte[4];
            byte[] new4Bytes = new byte[4] { 0x11, 0x22, 0x33, 0x44 };

            // try to read the whole user memory bank
            Console.WriteLine("\t -> 3, Trying to Read all bytes from user mem");
            if (ph.readFromTag(epcList[index], memBank, 0, passwd, 0, out userMemBefore))
            {
                Console.WriteLine("\t\t Read succeeded, read data:");
                Console.Write("\t\t\t ");
                for (int i = 0; i < userMemBefore.Length; i++)
                {
                    Console.Write("{0:X02}-", userMemBefore[i]);
                    if(((i + 1)%8) == 0)
                        Console.Write("\n\t\t\t ");
                }
                Console.Write("\n");
            }
            else
            {
                Console.WriteLine("\t\t Read failed: " + ph.LastReturnCode.ToString());
                return;
            }
            Console.WriteLine("");

            // save the old first 4 bytes of the user memory
            old4Bytes[0] = userMemBefore[0];
            old4Bytes[1] = userMemBefore[1];
            old4Bytes[2] = userMemBefore[2];
            old4Bytes[3] = userMemBefore[3];

            // try to write 4 dummy bytes to the user mem
            Console.WriteLine("\t -> 4, Trying to Wrtie data:");
            Console.WriteLine("\t\t\t {0}", BitConverter.ToString(new4Bytes));
            if (ph.writeToTag(epcList[index], memBank, 0, passwd, new4Bytes))
            {
                Console.WriteLine("\t\t Wrtie succeeded");
            }
            else
            {
                Console.WriteLine("\t\t Wrtie failed: " + ph.LastReturnCode.ToString());
                return;
            }
            Console.WriteLine("");

            // try to read the whole user me again
            Console.WriteLine("\t -> 5, Trying to Read written data");
            if (ph.readFromTag(epcList[index], memBank, 0, passwd, 0, out userMemAfter))
            {
                Console.WriteLine("\t\t Read succeeded, read data:");
                Console.Write("\t\t\t ");
                for (int i = 0; i < userMemAfter.Length; i++)
                {
                    Console.Write("{0:X02}-", userMemAfter[i]);
                    if (((i + 1) % 8) == 0)
                        Console.Write("\n\t\t\t ");
                }
                Console.Write("\n");
            }
            else
            {
                Console.WriteLine("\t\t Read failed: " + ph.LastReturnCode.ToString());
                return;
            }
            Console.WriteLine("");

            // try to write the old bytes into the memory
            Console.WriteLine("\t -> 6, Trying to restore the bytes");
            if (ph.writeToTag(epcList[index], memBank, 0, passwd, userMemBefore))
            {
                Console.WriteLine("\t\t Wrtie succeeded");
            }
            else
            {
                Console.WriteLine("\t\t Wrtie failed: " + ph.LastReturnCode.ToString());
                return;
            }
            Console.WriteLine("");

            // again read the restored bytes from the tag
            Console.WriteLine("\t -> 7, Trying to Read written data");
            if (ph.readFromTag(epcList[index], memBank, 0, passwd, 4, out userMemAfter))
            {
                Console.WriteLine("\t\t Read succeeded, read data: ");
                Console.Write("\t\t\t {0}", BitConverter.ToString(userMemAfter));
            }
            else
            {
                Console.WriteLine("\t\t Read failed: " + ph.LastReturnCode.ToString());
                return;
            }
            Console.WriteLine("");
        }


        #region AN

        public static void test_AN001_ReadTIDFirstTag_Slow(CSrfeProtocolHandler ph)
        {

            CSrfePURprotocolHandler pur_ph = _ph as CSrfePURprotocolHandler;
            if (pur_ph == null)
            {
                Console.WriteLine("Unable to test. Protocol implementation is not valid. Should be PUR.");
                return;
            }

            Console.WriteLine("Trying to read tid register:");

            storedTagIdSema = new Semaphore(0,1);
            storedTagId = null;
            CSrfeProtocolHandler.CyclicInventoryHandler newEventHandler = new CSrfeProtocolHandler.CyclicInventoryHandler(onCyclicInventoryStoreTagId);
            pur_ph.CyclicInventory += newEventHandler;

            Console.WriteLine("\t -> 1, Searching for tags:");

            tagReadCount = 0;
            pur_ph.setCyclicInventory(true);

            storedTagIdSema.WaitOne(5000);
            DateTime startTime = DateTime.Now;

            pur_ph.setCyclicInventory(false);

            if (storedTagId == null)
            {
                Console.WriteLine("\t\t Did not find any tag...");
                pur_ph.CyclicInventory -= newEventHandler;
                return;
            }
            
            Console.WriteLine("\t\t Found Tags {0}", BitConverter.ToString(storedTagId));


            // try to read the first 4 bytes of the TID membank of the detected tag
            Console.WriteLine("\t -> 2, Trying to read first 4 bytes of TID register:");
            byte[] passwd = new byte[4] { 0, 0, 0, 0 };
            byte[] data;
            if (pur_ph.readFromTag(storedTagId, (byte)0x02, 0, passwd, 4, out data))
            {
                Console.WriteLine("\t\t Read succeeded, read data: {0}", BitConverter.ToString(data));
            }
            else
            {
                Console.WriteLine("\t\t Read failed");
            }
            Console.WriteLine("");
            Console.WriteLine("Read Result {0}ms after the tag was detected", DateTime.Now.Subtract(startTime).TotalMilliseconds);
            Console.WriteLine("");

            pur_ph.CyclicInventory -= newEventHandler;
        }

        public static void test_AN001_ReadTIDFirstTag_Fast(CSrfeProtocolHandler ph)
        {
            CSrfePURprotocolHandler pur_ph = _ph as CSrfePURprotocolHandler;
            if (pur_ph == null)
            {
                Console.WriteLine("Unable to test. Protocol implementation is not valid. Should be PUR.");
                return;
            }

            Console.WriteLine("Trying to read tid register:");

            storedTagIdSema = new Semaphore(0, 1);
            storedTagId = null;
            CSrfeProtocolHandler.CyclicInventoryHandler newEventHandler = new CSrfeProtocolHandler.CyclicInventoryHandler(onCyclicInventoryStoreTagId);
            pur_ph.CyclicInventory += newEventHandler;

            Console.WriteLine("\t -> 1, Searching for tags:");

            tagReadCount = 0;
            pur_ph.setCyclicInventory(true);

            storedTagIdSema.WaitOne(5000);
            DateTime startTime = DateTime.Now;
            pur_ph.BlockCyclicInventoryInterrupts = true;

            if (storedTagId == null)
            {
                Console.WriteLine("\t\t Did not find any tag...");
                pur_ph.setCyclicInventory(false);
                pur_ph.CyclicInventory -= newEventHandler;
                pur_ph.BlockCyclicInventoryInterrupts = false;
                return;
            }


            Console.WriteLine("\t\t Found Tags {0}", BitConverter.ToString(storedTagId));

            // try to read the first 4 bytes of the TID membank of the detected tag
            Console.WriteLine("\t -> 2, Trying to read first 4 bytes of TID register:");
            byte[] passwd = new byte[4] { 0, 0, 0, 0 };
            byte[] data;
            if (pur_ph.readFromTag(storedTagId, (byte)0x02, 0, passwd, 4, out data))
            {
                Console.WriteLine("\t\t Read succeeded, read data: {0}", BitConverter.ToString(data));
            }
            else
            {
                Console.WriteLine("\t\t Read failed");
            }
            Console.WriteLine("");
            Console.WriteLine("Read Result {0}ms after the tag was detected", DateTime.Now.Subtract(startTime).TotalMilliseconds);
            Console.WriteLine("");

            pur_ph.setCyclicInventory(false);
            pur_ph.CyclicInventory -= newEventHandler;
            pur_ph.BlockCyclicInventoryInterrupts = false;
        }

        public static void test_AN001_ReadTIDFirstTag_Handle(CSrfeProtocolHandler ph)
        {
            CSrfePURprotocolHandler pur_ph = _ph as CSrfePURprotocolHandler;
            if (pur_ph == null)
            {
                Console.WriteLine("Unable to test. Protocol implementation is not valid. Should be PUR.");
                return;
            }

            Console.WriteLine("Trying to read tid register from first tag:");

            pur_ph.setParam(0x24, new byte[1] { 0x01 });        //send Handle ON
            pur_ph.setParam(0x00, new byte[1] { 0x04 });        //inventory_secure_single_tag
            pur_ph.setParam(0x03, new byte[1] { 0x02 });        //sendTagIDImmediatelyAndStop

            storedTagIdSema = new Semaphore(0, 1);
            storedTagEvent = null;
            CSrfeProtocolHandler.CyclicInventoryHandler newEventHandler = new CSrfeProtocolHandler.CyclicInventoryHandler(onCyclicInventoryStoreTagId);
            pur_ph.CyclicInventory += newEventHandler;

            Console.WriteLine("\t -> 1, Searching for tags and reading from first");

            pur_ph.setAntennaPower(true);

            tagReadCount = 0;
            pur_ph.setCyclicInventory(true);

            storedTagIdSema.WaitOne(5000);
            DateTime startTime = DateTime.Now;
            pur_ph.BlockCyclicInventoryInterrupts = true;

            if (storedTagEvent == null || storedTagEvent.hasHandle == false)
            {
                Console.WriteLine("\t\t Did not find any tag...");
                pur_ph.setCyclicInventory(false);
                pur_ph.setAntennaPower(false);
                pur_ph.setParam(0x24, new byte[1] { 0x00 });        //send Handle OFF
                pur_ph.setParam(0x00, new byte[2] { 0x00, 0x02 });  //inventory_secure_multi_tag
                pur_ph.setParam(0x03, new byte[1] { 0x00 });        //sendTagIDImmediately

                pur_ph.CyclicInventory -= newEventHandler;
                pur_ph.BlockCyclicInventoryInterrupts = false;
                return;
            }

            Console.WriteLine("\t\t Found Tags {0}", BitConverter.ToString(storedTagEvent.tagId));

            // try to read the first 4 bytes of the TID membank of the detected tag
            Console.WriteLine("\t -> 2, Trying to read first 4 bytes of TID register:");
            byte[] passwd = new byte[4] { 0, 0, 0, 0 };
            byte[] data;
            if (pur_ph.readFromHandle(storedTagEvent.handle, (byte)0x02, 0, passwd, 4, out data))
            {
                Console.WriteLine("\t\t Read succeeded, read data: {0}", BitConverter.ToString(data));
            }
            else
            {
                Console.WriteLine("\t\t Read failed");
            }
            Console.WriteLine("");
            Console.WriteLine("Read Result {0}ms after the tag was detected", DateTime.Now.Subtract(startTime).TotalMilliseconds);
            Console.WriteLine("");

            pur_ph.setCyclicInventory(false);
            pur_ph.setAntennaPower(false);
            pur_ph.setParam(0x24, new byte[1] { 0x00 });        //send Handle OFF
            pur_ph.setParam(0x00, new byte[1] { 0x02 });        //inventory_secure_multi_tag
            pur_ph.setParam(0x03, new byte[1] { 0x00 });        //sendTagIDImmediately
            pur_ph.CyclicInventory -= newEventHandler;
            pur_ph.BlockCyclicInventoryInterrupts = false;
        }

        #endregion

        #endregion

        #endregion Test Methods


        #region Event Handler

        private static DateTime lastHeartbeat = DateTime.Now;

        // event handler for the heartbeat event
        public static void onHeratBeat( byte[] data )
        {
            Console.WriteLine("\t\t Heartbeat after {0} ms / data: {1}", DateTime.Now.Subtract(lastHeartbeat).TotalMilliseconds, BitConverter.ToString(data));
            lastHeartbeat = DateTime.Now;
        }

        // event handler for the state changed event
        public static void onStateChanged(Constants.eRFE_CURRENT_READER_STATE newState)
        {
            Console.WriteLine("\t\t =============== State changed to: " + newState.ToString() + " ===============");
        }

        // event handler for the state changed event
        public static void onGpioValuesChanged(ulong gpioValues)
        {
            Console.WriteLine("\t\t GPIO Values Changed: = 0x{0:X08}", gpioValues);
        }


        private static ulong tagReadCount = 0;

        // event handler for cyclic inventory event
        public static void onCyclicInventory(CSrfeProtocolHandler.TagEvent tagEvent)
        {
            Console.WriteLine("\t\t{0:D8}  {1}", ++tagReadCount, BitConverter.ToString(tagEvent.tagId));

            if (tagEvent.hasRSSI)
            {
                Console.Write("\t\t\t\tQ: ");
                for (int i = 0; i < tagEvent.rssi[0]; i++)
                    Console.Write("#");
                Console.Write("\n");

                Console.Write("\t\t\t\tI: ");
                for (int i = 0; i < tagEvent.rssi[1]; i++)
                    Console.Write("#");
                Console.Write("\n");
            }
        }


        private static Semaphore    storedTagIdSema = null;
        private static byte[]       storedTagId = null;
        private static CSrfeProtocolHandler.TagEvent storedTagEvent = null;
        // event handler for cyclic inventory event
        public static void onCyclicInventoryStoreTagId(CSrfeProtocolHandler.TagEvent tagEvent)
        {
            if (storedTagId == null)
            {
                storedTagId = new byte[tagEvent.tagId.Length];
                tagEvent.tagId.CopyTo(storedTagId, 0);
            }

            if (storedTagEvent == null)
            {
                storedTagEvent = new CSrfeProtocolHandler.TagEvent(tagEvent);
            }

            storedTagIdSema.Release();
        }



        #endregion Event Handler
    }
}
