﻿/*
 * Copyright (c) 2008-2013, RF-Embedded GmbH
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO.Ports;

namespace Test
{
    public class SerialDevice : CSrfeReaderInterface.device.IProtocolDeviceInterface
    {
        private SerialPort  m_port;
 

        public SerialDevice(string portName)
        {
            // Create a new SerialPort object with default settings.
            m_port = new SerialPort();

            // Allow the user to set the appropriate properties.
            m_port.PortName = portName;
            m_port.BaudRate = 115200;
            m_port.Parity = Parity.None;
            m_port.DataBits = 8;
            m_port.StopBits = StopBits.One;
            m_port.Handshake = Handshake.None;

//            // Set the read/write timeouts
//            m_port.ReadTimeout = 1; // TODO: test -> default -1
//            m_port.WriteTimeout = 1; // TODO: test -> default -1

            m_port.DataReceived += new SerialDataReceivedEventHandler(DataReceived);

        }

        public override bool Open()
        {
            m_port.Open();
            return m_port.IsOpen;
        }

        public override bool Close()
        {
            m_port.Close();
            return !m_port.IsOpen;
        }

        public override bool Send(byte[] data)
        {
            try
            {
                m_port.Write(data, 0, data.Length);
            }
            catch (TimeoutException)
            {
                return false;
            }

            return true;
        }

        private void DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            byte[] buffer = new byte[m_port.BytesToRead];
            m_port.Read(buffer, 0, buffer.Length);
            RaiseDataReadEvent(buffer);
        }
    }
}
