/*
 * Copyright (c) 2008-2013, RF-Embedded GmbH
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QRFEMESSAGEQUEUE_H
#define QRFEMESSAGEQUEUE_H

#include <QObject>
#include <QMap>
#include <QMutex>
#include <QWaitCondition>

namespace QrfeReaderInterface
{
    /**
     * @brief The QrfeMessageQueue class enqueues messages from reader and has the possibility to wait for it.
     */
    class QrfeMessageQueue : public QObject
    {
        Q_OBJECT

    public:
        QrfeMessageQueue(QObject* parent = 0);
        virtual ~QrfeMessageQueue();

        void 		clearMessage (int id);
        QByteArray 	waitForMessage (int id, uint msecs, bool *ok);
        void 		enqueueMessage (int id, const QByteArray& message);

    private:
        QMap<int, QByteArray>	m_messages;
        QMutex*					m_messageMutex;

        QMutex 					m_waitMutex;
        QWaitCondition			m_messageAvailable;
    };
}

#endif // QRFEMESSAGEQUEUE_H
