/*
 * Copyright (c) 2008-2013, RF-Embedded GmbH
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "QrfePURprotocolHandler.h"
#include "../QrfeReaderInterface.h"

using namespace QrfeReaderInterface;

/**
 * @brief QrfePURprotocolHandler::QrfePURprotocolHandler constructs a PUR implementation of the protocol handler
 * @param device    The communication device
 * @param parent    Parent object
 */
QrfePURprotocolHandler::QrfePURprotocolHandler(QIODevice *device, QObject* parent)
    : QrfeProtocolHandler(device, parent)
{
}

/**
 * @brief QrfePURprotocolHandler::readFromHandle reads data from a tag using the handle
 * @param handle    Handle of the tag
 * @param mem_bank  Memory bank where to read data from
 * @param address   Address within the memory bank
 * @param passwd    The access password to read from the tag
 * @param count     The count of data that should be read
 * @param data      The read data
 * @return Succes of the operation
 */
bool QrfePURprotocolHandler::readFromHandle ( QByteArray handle, uchar mem_bank, ushort address, QByteArray passwd, uchar count, QByteArray& data )
{
    if (passwd.size() != 4){
        Global::trc(1, "Read from handle - NOK - Data");
        return false;
    }

    QByteArray payload;
    payload.append(handle);

    payload.append((uchar) mem_bank);
    payload.append((uchar) (address >> 8));
    payload.append((uchar) address);
    payload.append(passwd);

    payload.append(count);

    QByteArray resultData;

    if (!customTagCommand((uchar)0x05, payload, resultData))
        return false;

    data = resultData.mid(1);

    return true;
}
